[libcamera-devel,v4,10/11] libcamera: pipeline_handler: Prepare Request
diff mbox series

Message ID 20211201142936.107405-11-jacopo@jmondi.org
State Superseded
Delegated to: Jacopo Mondi
Headers show
Series
  • libcamera: Add support for Fence
Related show

Commit Message

Jacopo Mondi Dec. 1, 2021, 2:29 p.m. UTC
Before queueing a request to the device, any synchronization fence from
the Request' framebuffers has to be waited on.

Connect the Request::Private::prepared signal to the function that
queues requests to the hardware and call Request::Private::prepare().

When the waiting request queue is inspected, verify if it has completed its
preparation phase and queue it to the device.

Signed-off-by: Jacopo Mondi <jacopo@jmondi.org>
Reviewed-by: Laurent Pinchart <laurent.pinchart@ideasonboard.com>
---
 src/libcamera/pipeline_handler.cpp | 37 +++++++++++++++++++++++-------
 1 file changed, 29 insertions(+), 8 deletions(-)

Patch
diff mbox series

diff --git a/src/libcamera/pipeline_handler.cpp b/src/libcamera/pipeline_handler.cpp
index 863d206a7ddd..5b5cb9c18dad 100644
--- a/src/libcamera/pipeline_handler.cpp
+++ b/src/libcamera/pipeline_handler.cpp
@@ -7,6 +7,7 @@ 
 
 #include "libcamera/internal/pipeline_handler.h"
 
+#include <chrono>
 #include <sys/sysmacros.h>
 
 #include <libcamera/base/log.h>
@@ -18,6 +19,7 @@ 
 
 #include "libcamera/internal/camera.h"
 #include "libcamera/internal/device_enumerator.h"
+#include "libcamera/internal/framebuffer.h"
 #include "libcamera/internal/media_device.h"
 #include "libcamera/internal/request.h"
 #include "libcamera/internal/tracepoints.h"
@@ -36,6 +38,8 @@ 
  * the REGISTER_PIPELINE_HANDLER() macro.
  */
 
+using namespace std::chrono_literals;
+
 namespace libcamera {
 
 LOG_DEFINE_CATEGORY(Pipeline)
@@ -323,10 +327,16 @@  bool PipelineHandler::hasPendingRequests(const Camera *camera) const
  * \param[in] request The request to queue
  *
  * This function queues a capture request to the pipeline handler for
  * processing. The request is first added to the internal list of queued
  * requests, and then passed to the pipeline handler with a call to
  * queueRequestDevice(). If the pipeline handler fails in queuing the request
  * to the hardware the request is cancelled.
  *
  * Keeping track of queued requests ensures automatic completion of all requests
  * when the pipeline handler is stopped with stop(). Request completion shall be
@@ -339,7 +349,11 @@  void PipelineHandler::queueRequest(Request *request)
 	LIBCAMERA_TRACEPOINT(request_queue, request);
 
 	waitingRequests_.push(request);
-	doQueueRequests();
+
+	request->_d()->prepared.connect(this, [this]() {
+						doQueueRequests();
+					});
+	request->_d()->prepare(300ms);
 }
 
 /**
@@ -355,6 +369,11 @@  void PipelineHandler::doQueueRequest(Request *request)
 
 	request->_d()->sequence_ = data->requestSequence_++;
 
+	if (request->_d()->cancelled_) {
+		completeRequest(request);
+		return;
+	}
+
 	int ret = queueRequestDevice(camera, request);
 	if (ret) {
 		request->_d()->cancel();
@@ -363,18 +382,20 @@  void PipelineHandler::doQueueRequest(Request *request)
 }
 
 /**
  * \brief Queue requests to the device
  *
  * Iterate the list of waiting requests and queue them to the device one
- * by one.
+ * by one if they have been prepared.
  */
 void PipelineHandler::doQueueRequests()
 {
 	while (!waitingRequests_.empty()) {
 		Request *request = waitingRequests_.front();
-		waitingRequests_.pop();
+		if (!request->_d()->prepared_)
+			break;
 
 		doQueueRequest(request);
+		waitingRequests_.pop();
 	}
 }
@@ -323,10 +327,16 @@  bool PipelineHandler::hasPendingRequests(const Camera *camera) const
  *
  * This function queues a capture request to the pipeline handler for
- * processing. The request is first added to the internal list of queued
- * requests, and then passed to the pipeline handler with a call to
- * queueRequestDevice(). If the pipeline handler fails in queuing the request
- * to the hardware the request is cancelled.
+ * processing. The request is first added to the internal list of waiting
+ * requests which have to be prepared to make sure they are ready for being
+ * queued to the pipeline handler.
+ *
+ * The queue of waiting requests is iterated and all prepared requests are
+ * passed to the pipeline handler in the same order they have been queued by
+ * calling this function.
+ *
+ * If a Request fails during the preparation phase or if the pipeline handler
+ * fails in queuing the request to the hardware the request is cancelled.
  *
  * Keeping track of queued requests ensures automatic completion of all requests
@@ -363,18 +382,20 @@  void PipelineHandler::doQueueRequest(Request *request)
 
 /**
- * \brief Queue requests to the device
+ * \brief Queue prepared requests to the device
  *
  * Iterate the list of waiting requests and queue them to the device one